@@ -2,7 +2,7 @@
  * vout_xvideo.c: Xvideo video output display method
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000, 2001 VideoLAN
- * $Id: vout_xvideo.c,v 1.12 2001/04/27 19:29:11 massiot Exp $
+ * $Id: vout_xvideo.c,v 1.25 2001/08/05 15:32:46 gbazin Exp $
  *
  * Authors: Shane Harper <shanegh@optusnet.com.au>
  *          Vincent Seguin <seguin@via.ecp.fr>
@@ -43,6 +43,10 @@
 #include <sys/ipc.h>
 #endif
 
+#ifndef WIN32
+#include <netinet/in.h>                               /* BSD: struct in_addr */
+#endif
+
 #include <sys/shm.h>                                   /* shmget(), shmctl() */
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -56,7 +60,6 @@
 #include "threads.h"
 #include "mtime.h"
 #include "tests.h"
-#include "modules.h"
 
 #include "video.h"
 #include "video_output.h"
@@ -71,6 +74,8 @@
 #include "stream_control.h"                 /* needed by input_ext-intf.h... */
 #include "input_ext-intf.h"
 
+#include "modules.h"
+#include "modules_export.h"
 
 #define GUID_YUV12_PLANAR 0x32315659
 
@@ -127,9 +132,8 @@ typedef struct vout_sys_s
     /* Mouse pointer properties */
     boolean_t           b_mouse_pointer_visible;
     mtime_t             i_time_mouse_last_moved; /* used to auto-hide pointer*/
-
-    /* Displaying fullscreen */
-    boolean_t           b_fullscreen;
+    Cursor              blank_cursor;                   /* the hidden cursor */
+    Pixmap              cursor_pixmap;
 
 } vout_sys_t;
 
@@ -166,7 +170,7 @@ static int  XVideoCreateShmImage     ( Display* dpy, int xv_port,
                                        int i_width, int i_height );
 static void XVideoDestroyShmImage    ( vout_thread_t *, XvImage *,
                                        XShmSegmentInfo * );
-static void XVideoSetMousePointer    ( const vout_thread_t * );
+static void X11ToggleMousePointer    ( vout_thread_t * );
 static void XVideoEnableScreenSaver  ( vout_thread_t * );
 static void XVideoDisableScreenSaver ( vout_thread_t * );
 /*static void XVideoSetAttribute       ( vout_thread_t *, char *, float );*/
@@ -202,12 +206,42 @@ void _M( vout_getfunctions )( function_list_t * p_function_list )
  *****************************************************************************/
 static int vout_Probe( probedata_t *p_data )
 {
+    Display *p_display;                                   /* display pointer */
+    char    *psz_display;
+
+    /* Open display, unsing 'vlc_display' or DISPLAY environment variable */
+    psz_display = XDisplayName( main_GetPszVariable(VOUT_DISPLAY_VAR, NULL) );
+    p_display = XOpenDisplay( psz_display );
+    if( p_display == NULL )                                         /* error */
+    {
+        intf_WarnMsg( 3, "vout: Xvideo cannot open display %s", psz_display );
+        intf_WarnMsg( 3, "vout: Xvideo not supported" );
+        return( 0 );
+    }
+    
+    if( !XVideoCheckForXv( p_display ) )
+    {
+        intf_WarnMsg( 3, "vout: Xvideo not supported" );
+        XCloseDisplay( p_display );
+        return( 0 );
+    }
+
+    if( XVideoGetPort( p_display ) < 0 )
+    {
+        intf_WarnMsg( 3, "vout: Xvideo not supported" );
+        XCloseDisplay( p_display );
+        return( 0 );
+    }
+
+    /* Clean-up everyting */
+    XCloseDisplay( p_display );
+
     if( TestMethod( VOUT_METHOD_VAR, "xvideo" ) )
     {
         return( 999 );
     }
 
-    return( 110 );
+    return( 150 );
 }
 
 /*****************************************************************************
@@ -220,6 +254,7 @@ static int vout_Probe( probedata_t *p_data )
 static int vout_Create( vout_thread_t *p_vout )
 {
     char *psz_display;
+    XColor cursor_color;
 
     /* Allocate structure */
     p_vout->p_sys = malloc( sizeof( vout_sys_t ) );
@@ -241,7 +276,7 @@ static int vout_Create( vout_thread_t *p_vout )
     }
     p_vout->p_sys->i_screen = DefaultScreen( p_vout->p_sys->p_display );
 
-    p_vout->p_sys->b_fullscreen
+    p_vout->b_fullscreen
         = main_GetIntVariable( VOUT_FULLSCREEN_VAR, VOUT_FULLSCREEN_DEFAULT );
     
     if( !XVideoCheckForXv( p_vout->p_sys->p_display ) )
@@ -252,6 +287,40 @@ static int vout_Create( vout_thread_t *p_vout )
         return( 1 );
     }
 
+    /* Check we have access to a video port */
+    if( (p_vout->p_sys->xv_port = XVideoGetPort(p_vout->p_sys->p_display)) <0 )
+    {
+        intf_ErrMsg( "vout error: cannot get XVideo port" );
+        XCloseDisplay( p_vout->p_sys->p_display );
+        free( p_vout->p_sys );
+        return 1;
+    }
+    intf_DbgMsg( "Using xv port %d" , p_vout->p_sys->xv_port );
+
+    /* Create blank cursor (for mouse cursor autohiding) */
+    p_vout->p_sys->b_mouse_pointer_visible = 1;
+    p_vout->p_sys->cursor_pixmap = XCreatePixmap( p_vout->p_sys->p_display,
+                                                  DefaultRootWindow(
+                                                     p_vout->p_sys->p_display),
+                                                  1, 1, 1 );
+    
+    XParseColor( p_vout->p_sys->p_display,
+                 XCreateColormap( p_vout->p_sys->p_display,
+                                  DefaultRootWindow(
+                                                    p_vout->p_sys->p_display ),
+                                  DefaultVisual(
+                                                p_vout->p_sys->p_display,
+                                                p_vout->p_sys->i_screen ),
+                                  AllocNone ),
+                 "black", &cursor_color );
+    
+    p_vout->p_sys->blank_cursor = XCreatePixmapCursor(
+                                      p_vout->p_sys->p_display,
+                                      p_vout->p_sys->cursor_pixmap,
+                                      p_vout->p_sys->cursor_pixmap,
+                                      &cursor_color,
+                                      &cursor_color, 1, 1 );    
+
     /* Spawn base window - this window will include the video output window,
      * but also command buttons, subtitles and other indicators */
     if( XVideoCreateWindow( p_vout ) )
@@ -262,9 +331,7 @@ static int vout_Create( vout_thread_t *p_vout )
         return( 1 );
     }
 
-    if( (p_vout->p_sys->xv_port = XVideoGetPort( p_vout->p_sys->p_display ))<0 )
-        return 1;
-    intf_DbgMsg( "Using xv port %d" , p_vout->p_sys->xv_port );
+    /* p_vout->pf_setbuffers( p_vout, NULL, NULL ); */
 
 #if 0
     /* XXX The brightness and contrast values should be read from environment
@@ -273,8 +340,6 @@ static int vout_Create( vout_thread_t *p_vout )
     XVideoSetAttribute( p_vout, "XV_CONTRAST",   0.5 );
 #endif
 
-    p_vout->p_sys->b_mouse_pointer_visible = 1;
-
     /* Disable screen saver and return */
     XVideoDisableScreenSaver( p_vout );
 
@@ -286,7 +351,7 @@ static int vout_Create( vout_thread_t *p_vout )
  *****************************************************************************/
 static int vout_Init( vout_thread_t *p_vout )
 {
-#ifdef SYS_DARWIN1_3
+#ifdef SYS_DARWIN
     /* FIXME : As of 2001-03-16, XFree4 for MacOS X does not support Xshm. */
     p_vout->p_sys->b_shm = 0;
 #endif
@@ -315,6 +380,13 @@ static void vout_End( vout_thread_t *p_vout )
  *****************************************************************************/
 static void vout_Destroy( vout_thread_t *p_vout )
 {
+    /* Restore cursor if it was blanked */
+    if( !p_vout->p_sys->b_mouse_pointer_visible )
+        X11ToggleMousePointer( p_vout );
+
+    /* Destroy blank cursor pixmap */
+    XFreePixmap( p_vout->p_sys->p_display, p_vout->p_sys->cursor_pixmap );
+
     XVideoEnableScreenSaver( p_vout );
     XVideoDestroyWindow( p_vout );
     XCloseDisplay( p_vout->p_sys->p_display );
@@ -336,12 +408,9 @@ static void vout_Destroy( vout_thread_t *p_vout )
 static int vout_Manage( vout_thread_t *p_vout )
 {
     XEvent      xevent;                                         /* X11 event */
-    boolean_t   b_toggle_fullscreen;               /* user wants full-screen */
     char        i_key;                                    /* ISO Latin-1 key */
     KeySym      x_key_symbol;
 
-    b_toggle_fullscreen = 0;
-
     /* Handle X11 events: ConfigureNotify events are parsed to know if the
      * output window's size changed, MapNotify and UnmapNotify to know if the
      * window is mapped (and if the display is useful), and ClientMessages
@@ -409,7 +478,7 @@ static int vout_Manage( vout_thread_t *p_vout )
                             break;
                         case 'f':
                         case 'F':
-                            b_toggle_fullscreen = 1;
+                            p_vout->i_changes |= VOUT_FULLSCREEN_CHANGE;
                             break;
                         case '0':
                             network_ChannelJoin( 0 );
@@ -480,13 +549,13 @@ static int vout_Manage( vout_thread_t *p_vout )
         else if( xevent.type == MotionNotify )
         {
             p_vout->p_sys->i_time_mouse_last_moved = mdate();
-            p_vout->p_sys->b_mouse_pointer_visible = 1; 
-            XVideoSetMousePointer( p_vout ); 
+            if( !p_vout->p_sys->b_mouse_pointer_visible )
+                X11ToggleMousePointer( p_vout ); 
         }
         /* Other event */
         else
         {
-            intf_WarnMsg( 1, "%p -> unhandled event type %d received",
+            intf_WarnMsg( 3, "%p -> unhandled event type %d received",
                          p_vout, xevent.type );
         }
     }
@@ -525,11 +594,11 @@ static int vout_Manage( vout_thread_t *p_vout )
         }
     }
 
-    if ( b_toggle_fullscreen )
+    if ( p_vout->i_changes & VOUT_FULLSCREEN_CHANGE )
     {
         intf_DbgMsg( "vout: changing full-screen status" );
 
-        p_vout->p_sys->b_fullscreen = !p_vout->p_sys->b_fullscreen;
+        p_vout->b_fullscreen = !p_vout->b_fullscreen;
 
         /* Get rid of the old window */
         XVideoDestroyWindow( p_vout );
@@ -569,8 +638,7 @@ static int vout_Manage( vout_thread_t *p_vout )
     if( p_vout->p_sys->b_mouse_pointer_visible &&
         mdate() - p_vout->p_sys->i_time_mouse_last_moved > 2000000 )
     {
-        p_vout->p_sys->b_mouse_pointer_visible = 0; 
-        XVideoSetMousePointer( p_vout );
+        X11ToggleMousePointer( p_vout );
     }
     
     return 0;
@@ -589,7 +657,9 @@ static void vout_Display( vout_thread_t *p_vout )
                    p_vout->p_rendered_pic->i_height;
 
     if( XVideoUpdateImgSizeIfRequired( p_vout ) )
+    {
         return;
+    }
 
     switch( p_vout->p_rendered_pic->i_type )
     {
@@ -614,7 +684,9 @@ static void vout_Display( vout_thread_t *p_vout )
     }
 
     if( b_draw )
+    {
         XVideoDisplay( p_vout );
+    }
 }
 
 static void vout_SetPalette( p_vout_thread_t p_vout,
@@ -642,13 +714,16 @@ static int XVideoUpdateImgSizeIfRequired( vout_thread_t *p_vout )
     if( p_vout->p_sys->i_image_width != i_img_width
             || p_vout->p_sys->i_image_height != i_img_height )
     {
+        if( p_vout->p_sys->i_image_width != 0
+             && p_vout->p_sys->i_image_height != 0 )
+        {
+            /* Destroy XvImage to change its size */
+            vout_End( p_vout );
+        }
+
         p_vout->p_sys->i_image_width  = i_img_width;
         p_vout->p_sys->i_image_height = i_img_height;
 
-        /* Destroy XvImage to change its size */
-        vout_End( p_vout );
-            /* Note: vout_End does nothing if no XvImage to destroy. */
-
         /* Create XvImage using XShm extension */
         if( XVideoCreateShmImage( p_vout->p_sys->p_display,
                                   p_vout->p_sys->xv_port,
@@ -666,7 +741,6 @@ static int XVideoUpdateImgSizeIfRequired( vout_thread_t *p_vout )
             (p_vout->p_sys->p_xvimage->data_size) /
             (p_vout->p_sys->p_xvimage->height);
 
-        /* vout_SetBuffers( p_vout, p_vout->p_sys->p_xvimage->data ); */
     }
 
     return( 0 );
@@ -685,15 +759,15 @@ static int XVideoCheckForXv( Display *dpy )
             return( 1 );
 
         case XvBadExtension:
-            intf_ErrMsg( "vout error: XvBadExtension" );
+            intf_WarnMsg( 3, "vout error: XvBadExtension" );
             return( 0 );
 
         case XvBadAlloc:
-            intf_ErrMsg( "vout error: XvBadAlloc" );
+            intf_WarnMsg( 3, "vout error: XvBadAlloc" );
             return( 0 );
 
         default:
-            intf_ErrMsg( "vout error: XvQueryExtension failed" );
+            intf_WarnMsg( 3, "vout error: XvQueryExtension failed" );
             return( 0 );
     }
 }
@@ -717,7 +791,7 @@ static int XVideoCreateWindow( vout_thread_t *p_vout )
 
     /* Set main window's size */
     /* If we're full screen, we're full screen! */
-    if( p_vout->p_sys->b_fullscreen )
+    if( p_vout->b_fullscreen )
     {
         p_vout->p_sys->i_window_width = DisplayWidth( p_vout->p_sys->p_display,
                                                       p_vout->p_sys->i_screen );
@@ -761,7 +835,7 @@ static int XVideoCreateWindow( vout_thread_t *p_vout )
                            CWBackPixel | CWEventMask,
                            &xwindow_attributes );
 
-    if ( p_vout->p_sys->b_fullscreen )
+    if ( p_vout->b_fullscreen )
     {
         prop = XInternAtom(p_vout->p_sys->p_display, "_MOTIF_WM_HINTS", False);
         mwmhints.flags = MWM_HINTS_DECORATIONS;
@@ -837,7 +911,7 @@ static int XVideoCreateWindow( vout_thread_t *p_vout )
                   ButtonPressMask | ButtonReleaseMask | 
                   PointerMotionMask );
 
-    if( p_vout->p_sys->b_fullscreen )
+    if( p_vout->b_fullscreen )
     {
         XSetInputFocus( p_vout->p_sys->p_display, p_vout->p_sys->window,
                         RevertToNone, CurrentTime );
@@ -863,8 +937,12 @@ static int XVideoCreateWindow( vout_thread_t *p_vout )
     XSelectInput( p_vout->p_sys->p_display, p_vout->p_sys->yuv_window,
                   ExposureMask );
 
-
-    XVideoSetMousePointer( p_vout );
+    /* If the cursor was formerly blank than blank it again */
+    if( !p_vout->p_sys->b_mouse_pointer_visible )
+    {
+        X11ToggleMousePointer( p_vout );
+        X11ToggleMousePointer( p_vout );
+    }
 
     return( 0 );
 }
@@ -915,13 +993,16 @@ static int XVideoCreateShmImage( Display* dpy, int xv_port,
                                                         0, 0 );
     p_shm_info->readOnly = False;
 
+#if 0
     /* Mark the shm segment to be removed when there will be no more
      * attachements, so it is automatic on process exit or after shmdt */
     shmctl( p_shm_info->shmid, IPC_RMID, 0 );
+#endif
 
     if( !XShmAttach( dpy, p_shm_info ) )
     {
         intf_ErrMsg( "vout error: XShmAttach failed" );
+        shmctl( p_shm_info->shmid, IPC_RMID, 0 );
         shmdt( p_shm_info->shmaddr );
         return( -1 );
     }
@@ -954,8 +1035,10 @@ static void XVideoDestroyShmImage( vout_thread_t *p_vout, XvImage *p_xvimage,
     XDestroyImage( p_ximage ); /* XXX */
 #endif
 
+    shmctl( p_shm_info->shmid, IPC_RMID, 0 );
+
     if( shmdt( p_shm_info->shmaddr ) )  /* detach shared memory from process */
-    {                                   /* also automatic freeing...         */
+    {
         intf_ErrMsg( "vout error: cannot detach shared memory (%s)",
                      strerror(errno) );
     }
@@ -1001,45 +1084,24 @@ void XVideoDisableScreenSaver( vout_thread_t *p_vout )
 }
 
 /*****************************************************************************
- * XVideoSetMousePointer: hide or show the mouse pointer
+ * X11ToggleMousePointer: hide or show the mouse pointer
  *****************************************************************************
  * This function hides the X pointer if requested.
  *****************************************************************************/
-void XVideoSetMousePointer( const vout_thread_t *p_vout )
+void X11ToggleMousePointer( vout_thread_t *p_vout )
 {
-    static Cursor blank_cursor;
-    static boolean_t b_created_blank_cursor = 0;
 
-    if( !p_vout->p_sys->b_mouse_pointer_visible )
+    if( p_vout->p_sys->b_mouse_pointer_visible )
     {
-        if( !b_created_blank_cursor )
-        {
-            XColor color;
-            Pixmap blank = XCreatePixmap( p_vout->p_sys->p_display,
-                               DefaultRootWindow(p_vout->p_sys->p_display),
-                               1, 1, 1 );
-
-            XParseColor( p_vout->p_sys->p_display,
-                         XCreateColormap( p_vout->p_sys->p_display,
-                                          DefaultRootWindow(
-                                                  p_vout->p_sys->p_display ),
-                                          DefaultVisual(
-                                                  p_vout->p_sys->p_display,
-                                                  p_vout->p_sys->i_screen ),
-                                          AllocNone ),
-                         "black", &color );
-
-            blank_cursor = XCreatePixmapCursor( p_vout->p_sys->p_display,
-                           blank, blank, &color, &color, 1, 1 );
-
-            b_created_blank_cursor = 1;
-        }
         XDefineCursor( p_vout->p_sys->p_display,
-                       p_vout->p_sys->window, blank_cursor );
+                       p_vout->p_sys->window,
+                       p_vout->p_sys->blank_cursor );
+        p_vout->p_sys->b_mouse_pointer_visible = 0;
     }
     else
     {
         XUndefineCursor( p_vout->p_sys->p_display, p_vout->p_sys->window );
+        p_vout->p_sys->b_mouse_pointer_visible = 1;
     }
 }
 
@@ -1066,7 +1128,7 @@ static void XVideoOutputCoords( const picture_t *p_pic, const boolean_t scale,
                 *h = win_w * 9 / 16;
                 break;
 
-	    case AR_221_1_PICTURE:
+            case AR_221_1_PICTURE:
                 *h = win_w * 100 / 221;
                 break;
 
@@ -1120,19 +1182,20 @@ static int XVideoGetPort( Display *dpy )
             break;
 
         case XvBadExtension:
-            intf_ErrMsg( "vout error: XvBadExtension for XvQueryAdaptors" );
+            intf_WarnMsg( 3, "vout error: XvBadExtension for XvQueryAdaptors" );
             return( -1 );
 
         case XvBadAlloc:
-            intf_ErrMsg( "vout error: XvBadAlloc for XvQueryAdaptors" );
+            intf_WarnMsg( 3, "vout error: XvBadAlloc for XvQueryAdaptors" );
             return( -1 );
 
         default:
-            intf_ErrMsg( "vout error: XvQueryAdaptors failed" );
+            intf_WarnMsg( 3, "vout error: XvQueryAdaptors failed" );
             return( -1 );
     }
 
     for( i=0; i < i_adaptors && xv_port == -1; ++i )
+    {
         if( ( adaptor_info[ i ].type & XvInputMask ) &&
             ( adaptor_info[ i ].type & XvImageMask ) )
         {
@@ -1144,24 +1207,37 @@ static int XVideoGetPort( Display *dpy )
             imageFormats = XvListImageFormats( dpy, port, &i_num_formats );
 
             for( i=0; i < i_num_formats && xv_port == -1; ++i )
+            {
                 if( imageFormats[ i ].id == GUID_YUV12_PLANAR )
+                {
                     xv_port = port;
+                }
+            }
 
             if( xv_port == -1 )
+            {
                 intf_WarnMsg( 3, "vout: XVideo image input port %d "
                         "does not support the YUV12 planar format which is "
-                        "currently required by the xvideo output plugin.",
+                        "currently required by the xvideo output plugin",
                         port );
+            }
 
             if( imageFormats )
+            {
                 XFree( imageFormats );
+            }
         }
+    }
 
     if( i_adaptors > 0 )
+    {
         XvFreeAdaptorInfo(adaptor_info);
+    }
 
     if( xv_port == -1 )
-        intf_ErrMsg( "vout error: didn't find a suitable Xvideo image input port." );
+    {
+        intf_WarnMsg( 3, "vout: no suitable Xvideo image input port" );
+    }
 
     return( xv_port );
 }
@@ -1178,7 +1254,10 @@ static void XVideoDisplay( vout_thread_t *p_vout )
 {
     int     i_dest_width, i_dest_height, i_dest_x, i_dest_y;
 
-    if( !p_vout->p_sys->p_xvimage ) return;
+    if( !p_vout->p_sys->p_xvimage )
+    {
+        return;
+    }
 
     XVideoOutputCoords( p_vout->p_rendered_pic, p_vout->b_scale,
                         p_vout->p_sys->i_window_width,
@@ -1193,12 +1272,15 @@ static void XVideoDisplay( vout_thread_t *p_vout )
                    p_vout->p_rendered_pic->i_width,
                    p_vout->p_rendered_pic->i_height,
                    0 /*dest_x*/, 0 /*dest_y*/, i_dest_width, i_dest_height,
-                   True );
+                   False );
 
-     XResizeWindow( p_vout->p_sys->p_display, p_vout->p_sys->yuv_window,
-             i_dest_width, i_dest_height );
-     XMoveWindow( p_vout->p_sys->p_display, p_vout->p_sys->yuv_window,
-             i_dest_x, i_dest_y );
+    XResizeWindow( p_vout->p_sys->p_display, p_vout->p_sys->yuv_window,
+                   i_dest_width, i_dest_height );
+    XMoveWindow( p_vout->p_sys->p_display, p_vout->p_sys->yuv_window,
+                 i_dest_x, i_dest_y );
+
+    /* Send the order to the X server */
+    XSync( p_vout->p_sys->p_display, False );
 }
 
 #if 0
