@@ -2,7 +2,7 @@
  * ac3_parse.c: ac3 parsing procedures
  *****************************************************************************
  * Copyright (C) 1999, 2000, 2001 VideoLAN
- * $Id: ac3_parse.c,v 1.19 2001/04/26 00:12:19 reno Exp $
+ * $Id: ac3_parse.c,v 1.23 2001/05/15 16:19:42 sam Exp $
  *
  * Authors: Michel Kaempf <maxx@via.ecp.fr>
  *          Aaron Holtzman <aholtzma@engr.uvic.ca>
@@ -23,24 +23,32 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
+/*****************************************************************************
+ * Preamble
+ *****************************************************************************/
 #include "defs.h"
 
+#include <string.h>                                              /* memset() */
+
 #include "config.h"
 #include "common.h"
 
 #include "threads.h"
 #include "mtime.h"
 
+#include "intf_msg.h"
+
 #include "stream_control.h"
 #include "input_ext-dec.h"
 
 #include "audio_output.h"
 
-#include "intf_msg.h"
+#include "ac3_imdct.h"
+#include "ac3_downmix.h"
 #include "ac3_decoder.h"
-#include "ac3_decoder_thread.h"
+#include "ac3_decoder_thread.h"                           /* ac3dec_thread_t */
 
-#include "ac3_internal.h"
+#include "ac3_internal.h"                                       /* EXP_REUSE */
 
 /* Misc LUT */
 static const u16 nfchans[] = { 2, 1, 2, 3, 3, 4, 4, 5 };
@@ -95,8 +103,10 @@ static const struct frmsize_s frmsizecod_tbl[] =
 static const int fscod_tbl[] = {48000, 44100, 32000};
 
 /* Some internal functions */
-void parse_bsi_stats (ac3dec_t * p_ac3dec);
-void parse_audblk_stats (ac3dec_t * p_ac3dec);
+#ifdef STATS
+static void parse_bsi_stats (ac3dec_t * p_ac3dec);
+static void parse_audblk_stats (ac3dec_t * p_ac3dec);
+#endif
 
 /* Parse a syncinfo structure */
 int ac3_sync_frame (ac3dec_t * p_ac3dec, ac3_sync_info_t * p_sync_info) 
@@ -776,7 +786,7 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
     }
     
 #ifdef STATS
-//    parse_audblk_stats(p_ac3dec);
+    parse_audblk_stats(p_ac3dec);
 #endif
     
     return 0;
@@ -804,11 +814,12 @@ void parse_auxdata (ac3dec_t * p_ac3dec)
     RemoveBits (&p_ac3dec->bit_stream,16);
 }
 
-void parse_bsi_stats (ac3dec_t * p_ac3dec) /*Some stats */
+#ifdef STATS
+static void parse_bsi_stats (ac3dec_t * p_ac3dec) /* Some stats */
 {  
     struct mixlev_s
     {
-   	    float clev;
+        float clev;
         char *desc;
     };
     static const char *service_ids[8] = 
@@ -833,10 +844,10 @@ void parse_bsi_stats (ac3dec_t * p_ac3dec) /*Some stats */
     
     if ( !i )
     {
-/*     	if ((p_ac3dec->bsi.acmod & 0x1) && (p_ac3dec->bsi.acmod != 0x1))
-       		printf("CentreMixLevel %s ",cmixlev_tbl[p_ac3dec->bsi.cmixlev].desc);
-       	if (p_ac3dec->bsi.acmod & 0x4)
-       		printf("SurMixLevel %s",smixlev_tbl[p_ac3dec->bsi.cmixlev].desc);
+/*      if ((p_ac3dec->bsi.acmod & 0x1) && (p_ac3dec->bsi.acmod != 0x1))
+               printf("CentreMixLevel %s ",cmixlev_tbl[p_ac3dec->bsi.cmixlev].desc);
+        if (p_ac3dec->bsi.acmod & 0x4)
+               printf("SurMixLevel %s",smixlev_tbl[p_ac3dec->bsi.cmixlev].desc);
  */
         intf_Msg ( "(ac3dec_parsebsi) %s %d.%d Mode",
                 service_ids[p_ac3dec->bsi.bsmod],
@@ -848,24 +859,24 @@ void parse_bsi_stats (ac3dec_t * p_ac3dec) /*Some stats */
         i = 0;
 }
 
-void parse_audblk_stats (ac3dec_t * p_ac3dec)
+static void parse_audblk_stats (ac3dec_t * p_ac3dec)
 {
     char *exp_strat_tbl[4] = {"R   ","D15 ","D25 ","D45 "};
     u32 i;
 
-	intf_ErrMsg ("(ac3dec_parseaudblk) ");
-	intf_ErrMsg ("%s ",p_ac3dec->audblk.cplinu ? "cpl on" : "cpl off");
-	intf_ErrMsg ("%s ",p_ac3dec->audblk.baie? "bai" : " ");
-	intf_ErrMsg ("%s ",p_ac3dec->audblk.snroffste? "snroffst" : " ");
-	intf_ErrMsg ("%s ",p_ac3dec->audblk.deltbaie? "deltba" : " ");
-	intf_ErrMsg ("%s ",p_ac3dec->audblk.phsflginu? "phsflg" : " ");
-	intf_ErrMsg ("(%s %s %s %s %s) ",exp_strat_tbl[p_ac3dec->audblk.chexpstr[0]],
-		exp_strat_tbl[p_ac3dec->audblk.chexpstr[1]],exp_strat_tbl[p_ac3dec->audblk.chexpstr[2]],
-		exp_strat_tbl[p_ac3dec->audblk.chexpstr[3]],exp_strat_tbl[p_ac3dec->audblk.chexpstr[4]]);
-	intf_ErrMsg ("[");
-	for(i=0;i<p_ac3dec->bsi.nfchans;i++)
-		intf_ErrMsg ("%1d",p_ac3dec->audblk.blksw[i]);
-	intf_ErrMsg ("]");
-
-	intf_ErrMsg ("\n");
+    intf_ErrMsg ("(ac3dec_parseaudblk) ");
+    intf_ErrMsg ("%s ",p_ac3dec->audblk.cplinu ? "cpl on" : "cpl off");
+    intf_ErrMsg ("%s ",p_ac3dec->audblk.baie? "bai" : " ");
+    intf_ErrMsg ("%s ",p_ac3dec->audblk.snroffste? "snroffst" : " ");
+    intf_ErrMsg ("%s ",p_ac3dec->audblk.deltbaie? "deltba" : " ");
+    intf_ErrMsg ("%s ",p_ac3dec->audblk.phsflginu? "phsflg" : " ");
+    intf_ErrMsg ("(%s %s %s %s %s) ",exp_strat_tbl[p_ac3dec->audblk.chexpstr[0]],
+           exp_strat_tbl[p_ac3dec->audblk.chexpstr[1]],exp_strat_tbl[p_ac3dec->audblk.chexpstr[2]],
+           exp_strat_tbl[p_ac3dec->audblk.chexpstr[3]],exp_strat_tbl[p_ac3dec->audblk.chexpstr[4]]);
+    intf_ErrMsg ("[");
+    for(i=0;i<p_ac3dec->bsi.nfchans;i++)
+            intf_ErrMsg ("%1d",p_ac3dec->audblk.blksw[i]);
+    intf_ErrMsg ("]");
 }
+#endif
+
