@@ -2,7 +2,7 @@
  * input_ext-dec.c: services to the decoders
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: input_ext-dec.c,v 1.14 2001/04/28 03:36:25 sam Exp $
+ * $Id: input_ext-dec.c,v 1.17 2001/07/17 09:48:08 massiot Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -39,8 +39,7 @@
 #include "stream_control.h"
 #include "input_ext-dec.h"
 #include "input_ext-intf.h"
-
-#include "input.h"
+#include "input_ext-plugins.h"
 
 /*****************************************************************************
  * InitBitstream: initialize a bit_stream_t structure
@@ -181,12 +180,18 @@ u32 UnalignedShowBits( bit_stream_t * p_bit_stream, unsigned int i_bits )
                      * of the packet in a temporary buffer, and we'll see
                      * later. */
                     int     i;
+
+                    /* sizeof(WORD_TYPE) - number of bytes to trash
+                     * from the last payload */
+                    int     j;
+
                     p_bit_stream->i_showbits_buffer = 0;
 
-                    for( i = 0; i < sizeof(WORD_TYPE) ; i++ )
+                    for( j = i = 0 ; i < sizeof(WORD_TYPE) ; i++ )
                     {
                         if( p_bit_stream->p_byte >= p_bit_stream->p_end )
                         {
+                            j = i;
                             p_bit_stream->pf_next_data_packet( p_bit_stream );
                         }
                         ((byte_t *)&p_bit_stream->i_showbits_buffer)[i] =
@@ -195,7 +200,8 @@ u32 UnalignedShowBits( bit_stream_t * p_bit_stream, unsigned int i_bits )
                     }
 
                     /* This is kind of kludgy. */
-                    p_bit_stream->p_data->p_payload_start += sizeof(WORD_TYPE);
+                    p_bit_stream->p_data->p_payload_start +=
+                                                         sizeof(WORD_TYPE) - j;
                     p_bit_stream->p_byte =
                         (byte_t *)&p_bit_stream->i_showbits_buffer;
                     p_bit_stream->p_end =
